@@ -63,7 +63,6 @@ if T.TYPE_CHECKING:
     from mesonbuild.environment import Environment
     from mesonbuild._typing import Protocol
     from concurrent.futures import Future
-    from mesonbuild.modules.python import PythonIntrospectionDict
 
     class CompilerArgumentType(Protocol):
         cross_file: str
@@ -290,6 +289,7 @@ class TestDef:
 failing_logs: T.List[str] = []
 print_debug = 'MESON_PRINT_TEST_OUTPUT' in os.environ
 under_ci = 'CI' in os.environ
+ci_is_github = 'GITHUB_ACTIONS' in os.environ
 raw_ci_jobname = os.environ.get('MESON_CI_JOBNAME', None)
 ci_jobname = raw_ci_jobname if raw_ci_jobname != 'thirdparty' else None
 do_debug = under_ci or print_debug
@@ -421,11 +421,16 @@ def log_text_file(logfile: T.TextIO, testdir: Path, result: TestResult) -> None:
 
 
 def _run_ci_include(args: T.List[str]) -> str:
+    header = f'Included file {args[0]}:'
+    footer = ''
+    if ci_is_github:
+        header = f'::group::==== {header} ===='
+        footer = '::endgroup::'
     if not args:
         return 'At least one parameter required'
     try:
         data = Path(args[0]).read_text(errors='ignore', encoding='utf-8')
-        return 'Included file {}:\n{}\n'.format(args[0], data)
+        return f'{header}\n{data}\n{footer}'
     except Exception:
         return 'Failed to open {}'.format(args[0])
 
@@ -655,16 +660,19 @@ def _run_test(test: TestDef,
         gen_args.extend(['--native-file', nativefile.as_posix()])
     if crossfile.exists():
         gen_args.extend(['--cross-file', crossfile.as_posix()])
-    (returncode, stdo, stde) = run_configure(gen_args, env=test.env, catch_exception=True)
+    inprocess, res = run_configure(gen_args, env=test.env, catch_exception=True)
+    returncode, stdo, stde = res
+    cmd = '(inprocess) $ ' if inprocess else '$ '
+    cmd += mesonlib.join_args(gen_args)
     try:
         logfile = Path(test_build_dir, 'meson-logs', 'meson-log.txt')
         with logfile.open(errors='ignore', encoding='utf-8') as fid:
-            mesonlog = fid.read()
+            mesonlog = '\n'.join((cmd, fid.read()))
     except Exception:
         mesonlog = no_meson_log_msg
     cicmds = run_ci_commands(mesonlog)
     testresult = TestResult(cicmds)
-    testresult.add_step(BuildStep.configure, stdo, stde, mesonlog, time.time() - gen_start)
+    testresult.add_step(BuildStep.configure, '\n'.join((cmd, stdo)), stde, mesonlog, time.time() - gen_start)
     output_msg = validate_output(test, stdo, stde)
     testresult.mlog += output_msg
     if output_msg:
